@@ -21,8 +21,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include <stdio.h>
-#include <string.h>
 #include <zebra.h>
 
 #include "memory.h"
@@ -82,9 +80,9 @@ area_match (struct list *left, struct list *right)
   struct area_addr *addr1, *addr2;
   struct listnode *node1, *node2;
 
-  LIST_LOOP (left, addr1, node1)
+  for (ALL_LIST_ELEMENTS_RO (left, node1, addr1))
   {
-    LIST_LOOP (right, addr2, node2)
+    for (ALL_LIST_ELEMENTS_RO (right, node2, addr2))
     {
       if (addr1->addr_len == addr2->addr_len &&
 	  !memcmp (addr1->area_addr, addr2->area_addr, (int) addr1->addr_len))
@@ -102,7 +100,7 @@ area_match (struct list *left, struct list *right)
  * return  0            the IIH's IP is not in the IS's subnetwork
  *         1            the IIH's IP is in the IS's subnetwork
  */
-int
+static int
 ip_same_subnet (struct prefix_ipv4 *ip1, struct in_addr *ip2)
 {
   u_char *addr1, *addr2;
@@ -144,9 +142,9 @@ ip_match (struct list *left, struct list *right)
   if ((left == NULL) || (right == NULL))
     return 0;
   
-  LIST_LOOP (left, ip1, node1)
+  for (ALL_LIST_ELEMENTS_RO (left, node1, ip1))
   {
-    LIST_LOOP (right, ip2, node2)
+    for (ALL_LIST_ELEMENTS_RO (right, node2, ip2))
     {
       if (ip_same_subnet (ip1, ip2))
 	{
@@ -194,7 +192,7 @@ authentication_check (struct isis_passwd *one, struct isis_passwd *theother)
 /*
  * Processing helper functions
  */
-void
+static void
 tlvs_to_adj_nlpids (struct tlvs *tlvs, struct isis_adjacency *adj)
 {
   int i;
@@ -214,13 +212,13 @@ tlvs_to_adj_nlpids (struct tlvs *tlvs, struct isis_adjacency *adj)
     }
 }
 
-void
+static void
 del_ip_addr (void *val)
 {
   XFREE (MTYPE_ISIS_TMP, val);
 }
 
-void
+static void
 tlvs_to_adj_ipv4_addrs (struct tlvs *tlvs, struct isis_adjacency *adj)
 {
   struct listnode *node;
@@ -234,7 +232,7 @@ tlvs_to_adj_ipv4_addrs (struct tlvs *tlvs, struct isis_adjacency *adj)
   adj->ipv4_addrs = list_new ();
   if (tlvs->ipv4_addrs)
     {
-      LIST_LOOP (tlvs->ipv4_addrs, ipv4_addr, node)
+      for (ALL_LIST_ELEMENTS_RO (tlvs->ipv4_addrs, node, ipv4_addr))
       {
 	malloced = XMALLOC (MTYPE_ISIS_TMP, sizeof (struct in_addr));
 	memcpy (malloced, ipv4_addr, sizeof (struct in_addr));
@@ -244,7 +242,7 @@ tlvs_to_adj_ipv4_addrs (struct tlvs *tlvs, struct isis_adjacency *adj)
 }
 
 #ifdef HAVE_IPV6
-void
+static void
 tlvs_to_adj_ipv6_addrs (struct tlvs *tlvs, struct isis_adjacency *adj)
 {
   struct listnode *node;
@@ -258,7 +256,7 @@ tlvs_to_adj_ipv6_addrs (struct tlvs *tlvs, struct isis_adjacency *adj)
   adj->ipv6_addrs = list_new ();
   if (tlvs->ipv6_addrs)
     {
-      LIST_LOOP (tlvs->ipv6_addrs, ipv6_addr, node)
+      for (ALL_LIST_ELEMENTS_RO (tlvs->ipv6_addrs, node, ipv6_addr))
       {
 	malloced = XMALLOC (MTYPE_ISIS_TMP, sizeof (struct in6_addr));
 	memcpy (malloced, ipv6_addr, sizeof (struct in6_addr));
@@ -337,7 +335,7 @@ process_p2p_hello (struct isis_circuit *circuit)
   adj = circuit->u.p2p.neighbor;
   if (!adj)
     {
-      adj = isis_new_adj (hdr->source_id, (u_char *) "      ", 0, circuit);
+      adj = isis_new_adj (hdr->source_id, NULL, 0, circuit);
       if (adj == NULL)
 	return ISIS_ERROR;
       circuit->u.p2p.neighbor = adj;
@@ -884,7 +882,7 @@ process_lan_hello (int level, struct isis_circuit *circuit, u_char * ssnpa)
     {
       if (adj->adj_state != ISIS_ADJ_UP)
 	{
-	  LIST_LOOP (tlvs.lan_neighs, snpa, node)
+	  for (ALL_LIST_ELEMENTS_RO (tlvs.lan_neighs, node, snpa))
 	    if (!memcmp (snpa, circuit->u.bc.snpa, ETH_ALEN))
 	    {
 	      isis_adj_state_change (adj, ISIS_ADJ_UP,
@@ -903,7 +901,9 @@ out:
 		  circuit->area->area_tag,
 		  level, snpa_print (ssnpa), circuit->interface->name,
 		  circuit_t2string (circuit->circuit_is_type),
-		  circuit->circuit_id, stream_get_endp (circuit->rcv_stream));
+		  circuit->circuit_id,
+		  /* FIXME: use %z when we stop supporting old compilers. */
+		  (unsigned long) stream_get_endp (circuit->rcv_stream));
     }
 
   free_tlvs (&tlvs);
@@ -947,7 +947,9 @@ process_lsp (int level, struct isis_circuit *circuit, u_char * ssnpa)
 		  ntohl (hdr->seq_num),
 		  ntohs (hdr->checksum),
 		  ntohs (hdr->rem_lifetime),
-		  circuit->rcv_stream->endp, circuit->interface->name);
+		  /* FIXME: use %z when we stop supporting old compilers. */
+		  (unsigned long) stream_get_endp (circuit->rcv_stream), 
+		  circuit->interface->name);
     }
 
   assert (ntohs (hdr->pdu_len) > ISIS_LSP_HDR_LEN);
@@ -1079,7 +1081,8 @@ dontcheckadj:
 	      /* 7.3.16.4 b) 1)  */
 	      if (comp == LSP_NEWER)
 		{
-		  lsp_update (lsp, hdr, circuit->rcv_stream, circuit->area);
+		  lsp_update (lsp, hdr, circuit->rcv_stream, circuit->area,
+			      level);
 		  /* ii */
 		  ISIS_FLAGS_SET_ALL (lsp->SRMflags);
 		  /* iii */
@@ -1115,14 +1118,17 @@ dontcheckadj:
 		      && circuit->u.bc.is_dr[level - 1] == 1))
 		{
 		  lsp->lsp_header->seq_num = htonl (ntohl (hdr->seq_num) + 1);
-		  zlog_debug ("LSP LEN: %d", ntohs (lsp->lsp_header->pdu_len));
+		  if (isis->debugs & DEBUG_UPDATE_PACKETS)
+		    zlog_debug ("LSP LEN: %d",
+				ntohs (lsp->lsp_header->pdu_len));
 		  iso_csum_create (STREAM_DATA (lsp->pdu) + 12,
 				   ntohs (lsp->lsp_header->pdu_len) - 12, 12);
 		  ISIS_FLAGS_SET_ALL (lsp->SRMflags);
-		  zlog_debug
-		    ("ISIS-Upd (%s): (1) re-originating LSP %s new seq 0x%08x",
-		     circuit->area->area_tag, rawlspid_print (hdr->lsp_id),
-		     ntohl (lsp->lsp_header->seq_num));
+		  if (isis->debugs & DEBUG_UPDATE_PACKETS)
+		    zlog_debug ("ISIS-Upd (%s): (1) re-originating LSP %s new "
+				"seq 0x%08x", circuit->area->area_tag,
+				rawlspid_print (hdr->lsp_id),
+				ntohl (lsp->lsp_header->seq_num));
 		  lsp->lsp_header->rem_lifetime =
 		    htons (isis_jitter
 			   (circuit->area->max_lsp_lifetime[level - 1],
@@ -1162,10 +1168,11 @@ dontcheckadj:
 			   ntohs (lsp->lsp_header->pdu_len) - 12, 12);
 
 	  ISIS_FLAGS_SET_ALL (lsp->SRMflags);
-	  zlog_debug
-	    ("ISIS-Upd (%s): (2) re-originating LSP %s new seq 0x%08x",
-	     circuit->area->area_tag, rawlspid_print (hdr->lsp_id),
-	     ntohl (lsp->lsp_header->seq_num));
+	  if (isis->debugs & DEBUG_UPDATE_PACKETS)
+	    zlog_debug ("ISIS-Upd (%s): (2) re-originating LSP %s new seq "
+			"0x%08x", circuit->area->area_tag,
+			rawlspid_print (hdr->lsp_id),
+			ntohl (lsp->lsp_header->seq_num));
 	  lsp->lsp_header->rem_lifetime =
 	    htons (isis_jitter
 		   (circuit->area->max_lsp_lifetime[level - 1],
@@ -1229,7 +1236,7 @@ dontcheckadj:
       else if (comp == LSP_EQUAL)
 	{
 	  ISIS_CLEAR_FLAG (lsp->SRMflags, circuit);
-	  lsp_update (lsp, hdr, circuit->rcv_stream, circuit->area);
+	  lsp_update (lsp, hdr, circuit->rcv_stream, circuit->area, level);
 	  if (circuit->circ_type != CIRCUIT_T_BROADCAST)
 	    {
 	      ISIS_SET_FLAG (lsp->SSNflags, circuit);
@@ -1253,7 +1260,7 @@ dontcheckadj:
  * Section 7.3.15.2 - Action on receipt of a sequence numbers PDU
  */
 
-int
+static int
 process_snp (int snp_type, int level, struct isis_circuit *circuit,
 	     u_char * ssnpa)
 {
@@ -1267,7 +1274,8 @@ process_snp (int snp_type, int level, struct isis_circuit *circuit,
   uint32_t found = 0, expected = 0;
   struct isis_lsp *lsp;
   struct lsp_entry *entry;
-  struct listnode *node, *node2;
+  struct listnode *node, *nnode;
+  struct listnode *node2, *nnode2;
   struct tlvs tlvs;
   struct list *lsp_list = NULL;
   struct isis_passwd *passwd;
@@ -1424,7 +1432,7 @@ process_snp (int snp_type, int level, struct isis_circuit *circuit,
 		  typechar, snpa_print (ssnpa), circuit->interface->name);
       if (tlvs.lsp_entries)
 	{
-	  LIST_LOOP (tlvs.lsp_entries, entry, node)
+	  for (ALL_LIST_ELEMENTS_RO (tlvs.lsp_entries, node, entry))
 	  {
 	    zlog_debug ("ISIS-Snp (%s):         %cSNP entry %s, seq 0x%08x,"
 			" cksum 0x%04x, lifetime %us",
@@ -1440,7 +1448,7 @@ process_snp (int snp_type, int level, struct isis_circuit *circuit,
   /* 7.3.15.2 b) Actions on LSP_ENTRIES reported */
   if (tlvs.lsp_entries)
     {
-      LIST_LOOP (tlvs.lsp_entries, entry, node)
+      for (ALL_LIST_ELEMENTS_RO (tlvs.lsp_entries, node, entry))
       {
 	lsp = lsp_search (entry->lsp_id, circuit->area->lspdb[level - 1]);
 	own_lsp = !memcmp (entry->lsp_id, isis->sysid, ISIS_SYS_ID_LEN);
@@ -1507,9 +1515,9 @@ process_snp (int snp_type, int level, struct isis_circuit *circuit,
       /* Fixme: Find a better solution */
       if (tlvs.lsp_entries)
 	{
-	  LIST_LOOP (tlvs.lsp_entries, entry, node)
+	  for (ALL_LIST_ELEMENTS (tlvs.lsp_entries, node, nnode, entry))
 	  {
-	    LIST_LOOP (lsp_list, lsp, node2)
+	    for (ALL_LIST_ELEMENTS (lsp_list, node2, nnode2, lsp))
 	    {
 	      if (lsp_id_cmp (lsp->lsp_header->lsp_id, entry->lsp_id) == 0)
 		{
@@ -1520,7 +1528,7 @@ process_snp (int snp_type, int level, struct isis_circuit *circuit,
 	  }
 	}
       /* on remaining LSPs we set SRM (neighbor knew not of) */
-      LIST_LOOP (lsp_list, lsp, node2)
+      for (ALL_LIST_ELEMENTS_RO (lsp_list, node, lsp))
       {
 	ISIS_SET_FLAG (lsp->SRMflags, circuit);
       }
@@ -1532,7 +1540,7 @@ process_snp (int snp_type, int level, struct isis_circuit *circuit,
   return retval;
 }
 
-int
+static int
 process_csnp (int level, struct isis_circuit *circuit, u_char * ssnpa)
 {
   /* Sanity check - FIXME: move to correct place */
@@ -1546,7 +1554,7 @@ process_csnp (int level, struct isis_circuit *circuit, u_char * ssnpa)
   return process_snp (ISIS_SNP_CSNP_FLAG, level, circuit, ssnpa);
 }
 
-int
+static int
 process_psnp (int level, struct isis_circuit *circuit, u_char * ssnpa)
 {
   if ((stream_get_endp (circuit->rcv_stream) -
@@ -1568,7 +1576,7 @@ process_psnp (int level, struct isis_circuit *circuit, u_char * ssnpa)
  *           0x38	0x08	0x47	0x00	0x01	0x00	0x02	0x00
  *           0x03	0x00	0x81	0x01	0xcc
  */
-int
+static int
 process_is_hello (struct isis_circuit *circuit)
 {
   struct isis_adjacency *adj;
@@ -1588,7 +1596,7 @@ process_is_hello (struct isis_circuit *circuit)
   if (!adj)
     {
       /* 8.2.2 */
-      adj = isis_new_adj (sysid, (u_char *) "      ", 0, circuit);
+      adj = isis_new_adj (sysid, NULL, 0, circuit);
       if (adj == NULL)
 	return ISIS_ERROR;
 
@@ -1604,7 +1612,7 @@ process_is_hello (struct isis_circuit *circuit)
       /* 8.2.2 a) 2) delete the adj */
       XFREE (MTYPE_ISIS_ADJACENCY, adj);
       /* 8.2.2 a) 3) create a new adj */
-      adj = isis_new_adj (sysid, (u_char *) "      ", 0, circuit);
+      adj = isis_new_adj (sysid, NULL, 0, circuit);
       if (adj == NULL)
 	return ISIS_ERROR;
 
@@ -1636,7 +1644,7 @@ process_is_hello (struct isis_circuit *circuit)
  * PDU Dispatcher
  */
 
-int
+static int
 isis_handle_pdu (struct isis_circuit *circuit, u_char * ssnpa)
 {
   struct isis_fixed_hdr *hdr;
@@ -1875,7 +1883,7 @@ fill_fixed_hdr (struct isis_fixed_hdr *hdr, u_char pdu_type)
 /*
  * SEND SIDE                             
  */
-void
+static void
 fill_fixed_hdr_andstream (struct isis_fixed_hdr *hdr, u_char pdu_type,
 			  struct stream *stream)
 {
@@ -1938,7 +1946,7 @@ send_hello (struct isis_circuit *circuit, int level)
   hello_hdr.hold_time = htons ((u_int16_t) interval);
 
   hello_hdr.pdu_len = 0;	/* Update the PDU Length later */
-  len_pointer = stream_get_putp (circuit->snd_stream) + 3 + ISIS_SYS_ID_LEN;
+  len_pointer = stream_get_endp (circuit->snd_stream) + 3 + ISIS_SYS_ID_LEN;
 
   /* copy the shared part of the hello to the p2p hello if needed */
   if (circuit->circ_type == CIRCUIT_T_P2P)
@@ -2012,7 +2020,7 @@ send_hello (struct isis_circuit *circuit, int level)
     if (tlv_add_padding (circuit->snd_stream))
       return ISIS_WARNING;
 
-  length = stream_get_putp (circuit->snd_stream);
+  length = stream_get_endp (circuit->snd_stream);
   /* Update PDU length */
   stream_putw_at (circuit->snd_stream, len_pointer, (u_int16_t) length);
 
@@ -2027,20 +2035,22 @@ send_hello (struct isis_circuit *circuit, int level)
 	{
 	  zlog_debug ("ISIS-Adj (%s): Sent L%d LAN IIH on %s, length %ld",
 		      circuit->area->area_tag, level, circuit->interface->name,
-		      STREAM_SIZE (circuit->snd_stream));
+		      /* FIXME: use %z when we stop supporting old compilers. */
+		      (unsigned long) STREAM_SIZE (circuit->snd_stream));
 	}
       else
 	{
 	  zlog_debug ("ISIS-Adj (%s): Sent P2P IIH on %s, length %ld",
 		      circuit->area->area_tag, circuit->interface->name,
-		      STREAM_SIZE (circuit->snd_stream));
+		      /* FIXME: use %z when we stop supporting old compilers. */
+		      (unsigned long) STREAM_SIZE (circuit->snd_stream));
 	}
     }
 
   return retval;
 }
 
-int
+static int
 send_lan_hello (struct isis_circuit *circuit, int level)
 {
   return send_hello (circuit, level);
@@ -2111,7 +2121,7 @@ send_p2p_hello (struct thread *thread)
   return ISIS_OK;
 }
 
-int
+static int
 build_csnp (int level, u_char * start, u_char * stop, struct list *lsps,
 	    struct isis_circuit *circuit)
 {
@@ -2132,7 +2142,7 @@ build_csnp (int level, u_char * start, u_char * stop, struct list *lsps,
    * Fill Level 1 or 2 Complete Sequence Numbers header
    */
 
-  lenp = stream_get_putp (circuit->snd_stream);
+  lenp = stream_get_endp (circuit->snd_stream);
   stream_putw (circuit->snd_stream, 0);	/* PDU length - when we know it */
   /* no need to send the source here, it is always us if we csnp */
   stream_put (circuit->snd_stream, isis->sysid, ISIS_SYS_ID_LEN);
@@ -2159,7 +2169,7 @@ build_csnp (int level, u_char * start, u_char * stop, struct list *lsps,
     {
       retval = tlv_add_lsp_entries (lsps, circuit->snd_stream);
     }
-  length = (u_int16_t) stream_get_putp (circuit->snd_stream);
+  length = (u_int16_t) stream_get_endp (circuit->snd_stream);
   assert (length >= ISIS_CSNP_HDRLEN);
   /* Update PU length */
   stream_putw_at (circuit->snd_stream, lenp, length);
@@ -2200,9 +2210,10 @@ send_csnp (struct isis_circuit *circuit, int level)
       if (isis->debugs & DEBUG_SNP_PACKETS)
 	{
 	  zlog_debug ("ISIS-Snp (%s): Sent L%d CSNP on %s, length %ld",
-		      circuit->area->area_tag, level, circuit->interface->name,
-		      STREAM_SIZE (circuit->snd_stream));
-	  LIST_LOOP (list, lsp, node)
+		     circuit->area->area_tag, level, circuit->interface->name,
+		     /* FIXME: use %z when we stop supporting old compilers. */
+		     (unsigned long) STREAM_SIZE (circuit->snd_stream));
+	  for (ALL_LIST_ELEMENTS_RO (list, node, lsp))
 	  {
 	    zlog_debug ("ISIS-Snp (%s):         CSNP entry %s, seq 0x%08x,"
 			" cksum 0x%04x, lifetime %us",
@@ -2266,7 +2277,7 @@ send_l2_csnp (struct thread *thread)
   return retval;
 }
 
-int
+static int
 build_psnp (int level, struct isis_circuit *circuit, struct list *lsps)
 {
   struct isis_fixed_hdr fixed_hdr;
@@ -2287,7 +2298,7 @@ build_psnp (int level, struct isis_circuit *circuit, struct list *lsps)
   /*
    * Fill Level 1 or 2 Partial Sequence Numbers header
    */
-  lenp = stream_get_putp (circuit->snd_stream);
+  lenp = stream_get_endp (circuit->snd_stream);
   stream_putw (circuit->snd_stream, 0);	/* PDU length - when we know it */
   stream_put (circuit->snd_stream, isis->sysid, ISIS_SYS_ID_LEN);
   stream_putc (circuit->snd_stream, circuit->idx);
@@ -2313,7 +2324,7 @@ build_psnp (int level, struct isis_circuit *circuit, struct list *lsps)
 
   if (isis->debugs & DEBUG_SNP_PACKETS)
     {
-      LIST_LOOP (lsps, lsp, node)
+      for (ALL_LIST_ELEMENTS_RO (lsps, node, lsp))
       {
 	zlog_debug ("ISIS-Snp (%s):         PSNP entry %s, seq 0x%08x,"
 		    " cksum 0x%04x, lifetime %us",
@@ -2325,7 +2336,7 @@ build_psnp (int level, struct isis_circuit *circuit, struct list *lsps)
       }
     }
 
-  length = (u_int16_t) stream_get_putp (circuit->snd_stream);
+  length = (u_int16_t) stream_get_endp (circuit->snd_stream);
   assert (length >= ISIS_PSNP_HDRLEN);
   /* Update PDU length */
   stream_putw_at (circuit->snd_stream, lenp, length);
@@ -2337,7 +2348,7 @@ build_psnp (int level, struct isis_circuit *circuit, struct list *lsps)
  *  7.3.15.4 action on expiration of partial SNP interval
  *  level 1
  */
-int
+static int
 send_psnp (int level, struct isis_circuit *circuit)
 {
   int retval = ISIS_OK;
@@ -2368,7 +2379,9 @@ send_psnp (int level, struct isis_circuit *circuit)
 		zlog_debug ("ISIS-Snp (%s): Sent L%d PSNP on %s, length %ld",
 			    circuit->area->area_tag, level,
 			    circuit->interface->name,
-			    STREAM_SIZE (circuit->snd_stream));
+			    /* FIXME: use %z when we stop supporting old
+			     * compilers. */
+			    (unsigned long) STREAM_SIZE (circuit->snd_stream));
 
 	      retval = build_psnp (level, circuit, list);
 	      if (retval == ISIS_OK)
@@ -2380,11 +2393,8 @@ send_psnp (int level, struct isis_circuit *circuit)
 		   * sending succeeded, we can clear SSN flags of this circuit
 		   * for the LSPs in list
 		   */
-		  for (node = listhead (list); node; nextnode (node))
-		    {
-		      lsp = getdata (node);
-		      ISIS_CLEAR_FLAG (lsp->SSNflags, circuit);
-		    }
+		  for (ALL_LIST_ELEMENTS_RO (list, node, lsp))
+                    ISIS_CLEAR_FLAG (lsp->SSNflags, circuit);
 		}
 	    }
 	  list_delete (list);
@@ -2438,18 +2448,6 @@ send_l2_psnp (struct thread *thread)
   return retval;
 }
 
-void
-build_link_state (struct isis_lsp *lsp, struct isis_circuit *circuit,
-		  struct stream *stream)
-{
-  unsigned long length;
-
-  stream_put (stream, lsp->pdu, ntohs (lsp->lsp_header->pdu_len));
-  length = stream_get_putp (stream);
-
-  return;
-}
-
 /*
  * ISO 10589 - 7.3.14.3
  */
@@ -2466,10 +2464,7 @@ send_lsp (struct thread *thread)
 
   if (circuit->state == C_STATE_UP)
     {
-      node = listhead (circuit->lsp_queue);
-      assert (node);
-
-      lsp = getdata (node);
+      lsp = listgetdata ((node = listhead (circuit->lsp_queue)));
 
       /*
        * Do not send if levels do not match
@@ -2499,10 +2494,7 @@ send_lsp (struct thread *thread)
 		 circuit->interface->name);
 	    }
 	  /* copy our lsp to the send buffer */
-	  circuit->snd_stream->getp = lsp->pdu->getp;
-	  circuit->snd_stream->putp = lsp->pdu->putp;
-	  circuit->snd_stream->endp = lsp->pdu->endp;
-	  memcpy (circuit->snd_stream->data, lsp->pdu->data, lsp->pdu->endp);
+	  stream_copy (circuit->snd_stream, lsp->pdu);
 
 	  retval = circuit->tx (circuit, lsp->level);
 
@@ -2576,7 +2568,7 @@ ack_lsp (struct isis_link_state_hdr *hdr, struct isis_circuit *circuit,
 			      circuit->snd_stream);
 
 
-  lenp = stream_get_putp (circuit->snd_stream);
+  lenp = stream_get_endp (circuit->snd_stream);
   stream_putw (circuit->snd_stream, 0);	/* PDU length  */
   stream_put (circuit->snd_stream, isis->sysid, ISIS_SYS_ID_LEN);
   stream_putc (circuit->snd_stream, circuit->idx);
@@ -2588,7 +2580,7 @@ ack_lsp (struct isis_link_state_hdr *hdr, struct isis_circuit *circuit,
   stream_putl (circuit->snd_stream, ntohl (hdr->seq_num));
   stream_putw (circuit->snd_stream, ntohs (hdr->checksum));
 
-  length = (u_int16_t) stream_get_putp (circuit->snd_stream);
+  length = (u_int16_t) stream_get_endp (circuit->snd_stream);
   /* Update PDU length */
   stream_putw_at (circuit->snd_stream, lenp, length);
 
@@ -2597,91 +2589,3 @@ ack_lsp (struct isis_link_state_hdr *hdr, struct isis_circuit *circuit,
   return retval;
 }
 
-#if 0
-/*
- * ISH PDU Processing 
- */
-
-  /*
-   * Let's first check if the local and remote system have any common area
-   * addresses 
-   */
-if (area_match (tlvs.area_addrs, isis->man_area_addrs) == 0)
-  {
-    if (circuit->circuit_t == IS_LEVEL_2)
-      {
-	/* do as in table 8 (p. 40) */
-	switch (circuit_type)
-	  {
-	  case IS_LEVEL_1:
-	    if (adj->adj_state != ISIS_ADJ_UP)
-	      {
-		/* Reject */
-		zlog_warn ("areaMismatch");
-		retval = ISIS_WARNING;
-	      }
-	    else if (adj->adj_usage == ISIS_ADJ_LEVEL1)
-	      {
-		isis_adj_state_change (adj, ISIS_ADJ_DOWN, "Area Mismatch",
-				       circuit->adjdb);
-	      }
-	    else if (adj->adj_usage == ISIS_ADJ_LEVEL1AND2 ||
-		     adj->adj_usage == ISIS_ADJ_LEVEL2)
-	      {
-		isis_adj_state_change (adj, ISIS_ADJ_DOWN, "Wrong System",
-				       circuit->adjdb);
-	      }
-	    break;
-	  case IS_LEVEL_2:
-	    if (adj->adj_state != ISIS_ADJ_UP)
-	      {
-		isis_adj_state_change (adj, ISIS_ADJ_UP, NULL,
-				       circuit->adjdb);
-		adj->adj_usage = ISIS_ADJ_LEVEL2;
-	      }
-	    else if (adj->adj_usage == ISIS_ADJ_LEVEL1 ||
-		     adj->adj_usage == ISIS_ADJ_LEVEL1AND2)
-	      {
-		isis_adj_state_change (adj, ISIS_ADJ_DOWN, "Wrong System",
-				       circuit->adjdb);
-	      }
-	    else if (adj->adj_usage == ISIS_ADJ_LEVEL2)
-	      {
-		;		/* Accept */
-	      }
-	    break;
-	  case IS_LEVEL_1_AND_2:
-	    if (adj->adj_state != ISIS_ADJ_UP)
-	      {
-		isis_adj_state_change (adj, ISIS_ADJ_UP, NULL,
-				       circuit->adjdb);
-		adj->adj_usage = ISIS_ADJ_LEVEL2;
-	      }
-	    else if (adj->adj_usage == ISIS_ADJ_LEVEL1)
-	      {
-		isis_adj_state_change (adj, ISIS_ADJ_DOWN, "Wrong System",
-				       circuit->adjdb);
-	      }
-	    else if (adj->adj_usage == ISIS_ADJ_LEVEL1AND2)
-	      {
-		isis_adj_state_change (adj, ISIS_ADJ_DOWN, "Area Mismatch",
-				       circuit->adjdb);
-	      }
-	    else if (adj->adj_usage == ISIS_ADJ_LEVEL2)
-	      {
-		;		/* Accept */
-	      }
-	    break;
-	  }
-	goto mismatch;
-      }
-    else
-      {
-	isis_delete_adj (adj, circuit->adjdb);
-	zlog_warn ("areaMismatch");
-	return ISIS_WARNING;
-      }
-  }
-
-mismatch:
-#endif
