@@ -21,8 +21,6 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
 #include <zebra.h>
 
 #include "thread.h"
@@ -58,45 +56,20 @@ extern struct host host;
 int isis_run_spf_l1 (struct thread *thread);
 int isis_run_spf_l2 (struct thread *thread);
 
-/* performace issue ???? */
-void
-union_adjlist (struct list *target, struct list *source)
-{
-  struct isis_adjacency *adj, *adj2;
-  struct listnode *node, *node2;
-
-  zlog_debug ("Union adjlist!");
-  for (node = listhead (source); node; nextnode (node))
-    {
-      adj = getdata (node);
-
-      /* lookup adjacency in the source list */
-      for (node2 = listhead (target); node2; nextnode (node2))
-	{
-	  adj2 = getdata (node2);
-	  if (adj == adj2)
-	    break;
-	}
-
-      if (!node2)
-	listnode_add (target, adj);
-    }
-}
-
 /* 7.2.7 */
-void
+static void
 remove_excess_adjs (struct list *adjs)
 {
   struct listnode *node, *excess = NULL;
   struct isis_adjacency *adj, *candidate = NULL;
   int comp;
 
-  for (node = listhead (adjs); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (adjs, node, adj)) 
     {
       if (excess == NULL)
 	excess = node;
-      candidate = getdata (excess);
-      adj = getdata (node);
+      candidate = listgetdata (excess);
+
       if (candidate->sys_type < adj->sys_type)
 	{
 	  excess = node;
@@ -140,7 +113,8 @@ remove_excess_adjs (struct list *adjs)
   return;
 }
 
-const char *
+#ifdef EXTREME_DEBUG
+static const char *
 vtype2string (enum vertextype vtype)
 {
   switch (vtype)
@@ -148,9 +122,15 @@ vtype2string (enum vertextype vtype)
     case VTYPE_PSEUDO_IS:
       return "pseudo_IS";
       break;
+    case VTYPE_PSEUDO_TE_IS:
+      return "pseudo_TE-IS";
+      break;
     case VTYPE_NONPSEUDO_IS:
       return "IS";
       break;
+    case VTYPE_NONPSEUDO_TE_IS:
+      return "TE-IS";
+      break;
     case VTYPE_ES:
       return "ES";
       break;
@@ -160,6 +140,9 @@ vtype2string (enum vertextype vtype)
     case VTYPE_IPREACH_EXTERNAL:
       return "IP external";
       break;
+    case VTYPE_IPREACH_TE:
+      return "IP TE";
+      break;
 #ifdef HAVE_IPV6
     case VTYPE_IP6REACH_INTERNAL:
       return "IP6 internal";
@@ -174,20 +157,23 @@ vtype2string (enum vertextype vtype)
   return NULL;			/* Not reached */
 }
 
-const char *
+static const char *
 vid2string (struct isis_vertex *vertex, u_char * buff)
 {
   switch (vertex->type)
     {
     case VTYPE_PSEUDO_IS:
+    case VTYPE_PSEUDO_TE_IS:
       return rawlspid_print (vertex->N.id);
       break;
     case VTYPE_NONPSEUDO_IS:
+    case VTYPE_NONPSEUDO_TE_IS:
     case VTYPE_ES:
       return sysid_print (vertex->N.id);
       break;
     case VTYPE_IPREACH_INTERNAL:
     case VTYPE_IPREACH_EXTERNAL:
+    case VTYPE_IPREACH_TE:
 #ifdef HAVE_IPV6
     case VTYPE_IP6REACH_INTERNAL:
     case VTYPE_IP6REACH_EXTERNAL:
@@ -200,26 +186,26 @@ vid2string (struct isis_vertex *vertex, u_char * buff)
 
   return (char *) buff;
 }
+#endif /* EXTREME_DEBUG */
 
-struct isis_spftree *
+static struct isis_spftree *
 isis_spftree_new ()
 {
   struct isis_spftree *tree;
 
-  tree = XMALLOC (MTYPE_ISIS_SPFTREE, sizeof (struct isis_spftree));
+  tree = XCALLOC (MTYPE_ISIS_SPFTREE, sizeof (struct isis_spftree));
   if (tree == NULL)
     {
       zlog_err ("ISIS-Spf: isis_spftree_new Out of memory!");
       return NULL;
     }
-  memset (tree, 0, sizeof (struct isis_spftree));
 
   tree->tents = list_new ();
   tree->paths = list_new ();
   return tree;
 }
 
-void
+static void
 isis_vertex_del (struct isis_vertex *vertex)
 {
   list_delete (vertex->Adj_N);
@@ -229,7 +215,8 @@ isis_vertex_del (struct isis_vertex *vertex)
   return;
 }
 
-void
+#if 0 /* HT: Not used yet. */
+static void
 isis_spftree_del (struct isis_spftree *spftree)
 {
   spftree->tents->del = (void (*)(void *)) isis_vertex_del;
@@ -242,6 +229,7 @@ isis_spftree_del (struct isis_spftree *spftree)
 
   return;
 }
+#endif 
 
 void
 spftree_area_init (struct isis_area *area)
@@ -270,31 +258,33 @@ spftree_area_init (struct isis_area *area)
   return;
 }
 
-struct isis_vertex *
+static struct isis_vertex *
 isis_vertex_new (void *id, enum vertextype vtype)
 {
   struct isis_vertex *vertex;
 
-  vertex = XMALLOC (MTYPE_ISIS_VERTEX, sizeof (struct isis_vertex));
+  vertex = XCALLOC (MTYPE_ISIS_VERTEX, sizeof (struct isis_vertex));
   if (vertex == NULL)
     {
       zlog_err ("isis_vertex_new Out of memory!");
       return NULL;
     }
 
-  memset (vertex, 0, sizeof (struct isis_vertex));
   vertex->type = vtype;
   switch (vtype)
     {
     case VTYPE_ES:
     case VTYPE_NONPSEUDO_IS:
+    case VTYPE_NONPSEUDO_TE_IS:
       memcpy (vertex->N.id, (u_char *) id, ISIS_SYS_ID_LEN);
       break;
     case VTYPE_PSEUDO_IS:
+    case VTYPE_PSEUDO_TE_IS:
       memcpy (vertex->N.id, (u_char *) id, ISIS_SYS_ID_LEN + 1);
       break;
     case VTYPE_IPREACH_INTERNAL:
     case VTYPE_IPREACH_EXTERNAL:
+    case VTYPE_IPREACH_TE:
 #ifdef HAVE_IPV6
     case VTYPE_IP6REACH_INTERNAL:
     case VTYPE_IP6REACH_EXTERNAL:
@@ -314,7 +304,7 @@ isis_vertex_new (void *id, enum vertextype vtype)
 /*
  * Add this IS to the root of SPT
  */
-void
+static void
 isis_spf_add_self (struct isis_spftree *spftree, struct isis_area *area,
 		   int level)
 {
@@ -333,7 +323,11 @@ isis_spf_add_self (struct isis_spftree *spftree, struct isis_area *area,
   if (lsp == NULL)
     zlog_warn ("ISIS-Spf: could not find own l%d LSP!", level);
 
-  vertex = isis_vertex_new (isis->sysid, VTYPE_NONPSEUDO_IS);
+  if (!area->oldmetric)
+    vertex = isis_vertex_new (isis->sysid, VTYPE_NONPSEUDO_TE_IS);
+  else
+    vertex = isis_vertex_new (isis->sysid, VTYPE_NONPSEUDO_IS);
+
   vertex->lsp = lsp;
 
   listnode_add (spftree->paths, vertex);
@@ -347,31 +341,33 @@ isis_spf_add_self (struct isis_spftree *spftree, struct isis_area *area,
   return;
 }
 
-struct isis_vertex *
+static struct isis_vertex *
 isis_find_vertex (struct list *list, void *id, enum vertextype vtype)
 {
   struct listnode *node;
   struct isis_vertex *vertex;
   struct prefix *p1, *p2;
 
-  for (node = listhead (list); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (list, node, vertex))
     {
-      vertex = getdata (node);
       if (vertex->type != vtype)
 	continue;
       switch (vtype)
 	{
 	case VTYPE_ES:
 	case VTYPE_NONPSEUDO_IS:
+	case VTYPE_NONPSEUDO_TE_IS:
 	  if (memcmp ((u_char *) id, vertex->N.id, ISIS_SYS_ID_LEN) == 0)
 	    return vertex;
 	  break;
 	case VTYPE_PSEUDO_IS:
+	case VTYPE_PSEUDO_TE_IS:
 	  if (memcmp ((u_char *) id, vertex->N.id, ISIS_SYS_ID_LEN + 1) == 0)
 	    return vertex;
 	  break;
 	case VTYPE_IPREACH_INTERNAL:
 	case VTYPE_IPREACH_EXTERNAL:
+	case VTYPE_IPREACH_TE:
 #ifdef HAVE_IPV6
 	case VTYPE_IP6REACH_INTERNAL:
 	case VTYPE_IP6REACH_EXTERNAL:
@@ -392,9 +388,9 @@ isis_find_vertex (struct list *list, void *id, enum vertextype vtype)
 /*
  * Add a vertex to TENT sorted by cost and by vertextype on tie break situation
  */
-struct isis_vertex *
+static struct isis_vertex *
 isis_spf_add2tent (struct isis_spftree *spftree, enum vertextype vtype,
-		   void *id, struct isis_adjacency *adj, u_int16_t cost,
+		   void *id, struct isis_adjacency *adj, u_int32_t cost,
 		   int depth, int family)
 {
   struct isis_vertex *vertex, *v;
@@ -420,9 +416,11 @@ isis_spf_add2tent (struct isis_spftree *spftree, enum vertextype vtype,
       listnode_add (spftree->tents, vertex);
       return vertex;
     }
-  for (node = listhead (spftree->tents); node; nextnode (node))
+  
+  /* XXX: This cant use the standard ALL_LIST_ELEMENT macro */
+  for (node = listhead (spftree->tents); node; node = listnextnode (node))
     {
-      v = getdata (node);
+      v = listgetdata (node);
       if (v->d_N > vertex->d_N)
 	{
 	  list_add_node_prev (spftree->tents, node, vertex);
@@ -437,8 +435,9 @@ isis_spf_add2tent (struct isis_spftree *spftree, enum vertextype vtype,
 		{
 		  break;
 		}
-	      nextnode (node);
-	      (node) ? (v = getdata (node)) : (v = NULL);
+              /* XXX: this seems dubious, node is the loop iterator */
+	      node = listnextnode (node);
+	      (node) ? (v = listgetdata (node)) : (v = NULL);
 	    }
 	  list_add_node_prev (spftree->tents, node, vertex);
 	  break;
@@ -452,9 +451,9 @@ isis_spf_add2tent (struct isis_spftree *spftree, enum vertextype vtype,
   return vertex;
 }
 
-struct isis_vertex *
+static struct isis_vertex *
 isis_spf_add_local (struct isis_spftree *spftree, enum vertextype vtype,
-		    void *id, struct isis_adjacency *adj, u_int16_t cost,
+		    void *id, struct isis_adjacency *adj, u_int32_t cost,
 		    int family)
 {
   struct isis_vertex *vertex;
@@ -486,7 +485,7 @@ add2tent:
   return isis_spf_add2tent (spftree, vtype, id, adj, cost, 1, family);
 }
 
-void
+static void
 process_N (struct isis_spftree *spftree, enum vertextype vtype, void *id,
 	   u_int16_t dist, u_int16_t depth, struct isis_adjacency *adj,
 	   int family)
@@ -548,14 +547,16 @@ process_N (struct isis_spftree *spftree, enum vertextype vtype, void *id,
 /*
  * C.2.6 Step 1
  */
-int
+static int
 isis_spf_process_lsp (struct isis_spftree *spftree, struct isis_lsp *lsp,
-		      uint16_t cost, uint16_t depth, int family)
+		      uint32_t cost, uint16_t depth, int family)
 {
   struct listnode *node, *fragnode = NULL;
   u_int16_t dist;
   struct is_neigh *is_neigh;
+  struct te_is_neigh *te_is_neigh;
   struct ipv4_reachability *ipreach;
+  struct te_ipv4_reachability *te_ipv4_reach;
   enum vertextype vtype;
   struct prefix prefix;
 #ifdef HAVE_IPV6
@@ -580,10 +581,8 @@ lspfragloop:
     {
       if (lsp->tlv_data.is_neighs)
 	{
-	  for (node = listhead (lsp->tlv_data.is_neighs); node;
-	       nextnode (node))
+          for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.is_neighs, node, is_neigh))
 	    {
-	      is_neigh = getdata (node);
 	      /* C.2.6 a) */
 	      /* Two way connectivity */
 	      if (!memcmp (is_neigh->neigh_id, isis->sysid, ISIS_SYS_ID_LEN))
@@ -595,13 +594,28 @@ lspfragloop:
 			 depth + 1, lsp->adj, family);
 	    }
 	}
+      if (lsp->tlv_data.te_is_neighs)
+	{
+	  for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.te_is_neighs, node,
+				     te_is_neigh))
+	    {
+	      uint32_t metric;
+	      if (!memcmp (te_is_neigh->neigh_id, isis->sysid, ISIS_SYS_ID_LEN))
+		continue;
+	      memcpy (&metric, te_is_neigh->te_metric, 3);
+	      dist = cost + ntohl (metric << 8);
+	      vtype = LSP_PSEUDO_ID (te_is_neigh->neigh_id) ? VTYPE_PSEUDO_TE_IS
+		: VTYPE_NONPSEUDO_TE_IS;
+	      process_N (spftree, vtype, (void *) te_is_neigh->neigh_id, dist,
+			 depth + 1, lsp->adj, family);
+	    }
+	}
       if (family == AF_INET && lsp->tlv_data.ipv4_int_reachs)
 	{
 	  prefix.family = AF_INET;
-	  for (node = listhead (lsp->tlv_data.ipv4_int_reachs); node;
-	       nextnode (node))
+          for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.ipv4_int_reachs, 
+                                     node, ipreach))
 	    {
-	      ipreach = getdata (node);
 	      dist = cost + ipreach->metrics.metric_default;
 	      vtype = VTYPE_IPREACH_INTERNAL;
 	      prefix.u.prefix4 = ipreach->prefix;
@@ -614,10 +628,9 @@ lspfragloop:
       if (family == AF_INET && lsp->tlv_data.ipv4_ext_reachs)
 	{
 	  prefix.family = AF_INET;
-	  for (node = listhead (lsp->tlv_data.ipv4_ext_reachs); node;
-	       nextnode (node))
+          for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.ipv4_ext_reachs,
+                                     node, ipreach))
 	    {
-	      ipreach = getdata (node);
 	      dist = cost + ipreach->metrics.metric_default;
 	      vtype = VTYPE_IPREACH_EXTERNAL;
 	      prefix.u.prefix4 = ipreach->prefix;
@@ -626,14 +639,28 @@ lspfragloop:
 			 lsp->adj, family);
 	    }
 	}
+      if (family == AF_INET && lsp->tlv_data.te_ipv4_reachs)
+	{
+	  prefix.family = AF_INET;
+	  for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.te_ipv4_reachs,
+				     node, te_ipv4_reach))
+	    {
+	      dist = cost + ntohl (te_ipv4_reach->te_metric);
+	      vtype = VTYPE_IPREACH_TE;
+	      prefix.u.prefix4 = newprefix2inaddr (&te_ipv4_reach->prefix_start,
+						   te_ipv4_reach->control);
+	      prefix.prefixlen = (te_ipv4_reach->control & 0x3F);
+	      process_N (spftree, vtype, (void *) &prefix, dist, depth + 1,
+			 lsp->adj, family);
+	    }
+	}
 #ifdef HAVE_IPV6
       if (family == AF_INET6 && lsp->tlv_data.ipv6_reachs)
 	{
 	  prefix.family = AF_INET6;
-	  for (node = listhead (lsp->tlv_data.ipv6_reachs); node;
-	       nextnode (node))
+          for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.ipv6_reachs, 
+                                     node, ip6reach))
 	    {
-	      ip6reach = getdata (node);
 	      dist = cost + ip6reach->metric;
 	      vtype = (ip6reach->control_info & CTRL_INFO_DISTRIBUTION) ?
 		VTYPE_IP6REACH_EXTERNAL : VTYPE_IP6REACH_INTERNAL;
@@ -650,24 +677,25 @@ lspfragloop:
   if (fragnode == NULL)
     fragnode = listhead (lsp->lspu.frags);
   else
-    nextnode (fragnode);
+    fragnode = listnextnode (fragnode);
 
   if (fragnode)
     {
-      lsp = getdata (fragnode);
+      lsp = listgetdata (fragnode);
       goto lspfragloop;
     }
 
   return ISIS_OK;
 }
 
-int
+static int
 isis_spf_process_pseudo_lsp (struct isis_spftree *spftree,
 			     struct isis_lsp *lsp, uint16_t cost,
 			     uint16_t depth, int family)
 {
   struct listnode *node, *fragnode = NULL;
   struct is_neigh *is_neigh;
+  struct te_is_neigh *te_is_neigh;
   enum vertextype vtype;
 
 pseudofragloop:
@@ -679,42 +707,58 @@ pseudofragloop:
       return ISIS_WARNING;
     }
 
-  for (node = (lsp->tlv_data.is_neighs ?
-	       listhead (lsp->tlv_data.is_neighs) : NULL);
-       node; nextnode (node))
-    {
-      is_neigh = getdata (node);
-      vtype = LSP_PSEUDO_ID (is_neigh->neigh_id) ? VTYPE_PSEUDO_IS
-	: VTYPE_NONPSEUDO_IS;
-      /* Two way connectivity */
-      if (!memcmp (is_neigh->neigh_id, isis->sysid, ISIS_SYS_ID_LEN))
-	continue;
-      if (isis_find_vertex
-	  (spftree->tents, (void *) is_neigh->neigh_id, vtype) == NULL
-	  && isis_find_vertex (spftree->paths, (void *) is_neigh->neigh_id,
+  if (lsp->tlv_data.is_neighs)
+    for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.is_neighs, node, is_neigh))
+      {
+	vtype = LSP_PSEUDO_ID (is_neigh->neigh_id) ? VTYPE_PSEUDO_IS
+	  : VTYPE_NONPSEUDO_IS;
+	/* Two way connectivity */
+	if (!memcmp (is_neigh->neigh_id, isis->sysid, ISIS_SYS_ID_LEN))
+	  continue;
+	if (isis_find_vertex
+	    (spftree->tents, (void *) is_neigh->neigh_id, vtype) == NULL
+	    && isis_find_vertex (spftree->paths, (void *) is_neigh->neigh_id,
 			       vtype) == NULL)
-	{
-	  /* C.2.5 i) */
-	  isis_spf_add2tent (spftree, vtype, is_neigh->neigh_id, lsp->adj,
+	  {
+	    /* C.2.5 i) */
+	    isis_spf_add2tent (spftree, vtype, is_neigh->neigh_id, lsp->adj,
 			     cost, depth, family);
-	}
-    }
+	  }
+      }
+  if (lsp->tlv_data.te_is_neighs)
+    for (ALL_LIST_ELEMENTS_RO (lsp->tlv_data.te_is_neighs, node, te_is_neigh))
+      {
+	vtype = LSP_PSEUDO_ID (te_is_neigh->neigh_id) ? VTYPE_PSEUDO_TE_IS
+	  : VTYPE_NONPSEUDO_TE_IS;
+	/* Two way connectivity */
+	if (!memcmp (te_is_neigh->neigh_id, isis->sysid, ISIS_SYS_ID_LEN))
+	  continue;
+	if (isis_find_vertex
+	    (spftree->tents, (void *) te_is_neigh->neigh_id, vtype) == NULL
+	    && isis_find_vertex (spftree->paths, (void *) te_is_neigh->neigh_id,
+				 vtype) == NULL)
+	  {
+	    /* C.2.5 i) */
+	    isis_spf_add2tent (spftree, vtype, te_is_neigh->neigh_id, lsp->adj,
+			       cost, depth, family);
+	  }
+      }
 
   if (fragnode == NULL)
     fragnode = listhead (lsp->lspu.frags);
   else
-    nextnode (fragnode);
+    fragnode = listnextnode (fragnode);
 
   if (fragnode)
     {
-      lsp = getdata (fragnode);
+      lsp = listgetdata (fragnode);
       goto pseudofragloop;
     }
 
   return ISIS_OK;
 }
 
-int
+static int
 isis_spf_preload_tent (struct isis_spftree *spftree,
 		       struct isis_area *area, int level, int family)
 {
@@ -733,9 +777,8 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
   struct prefix_ipv6 *ipv6;
 #endif /* HAVE_IPV6 */
 
-  for (cnode = listhead (area->circuit_list); cnode; nextnode (cnode))
+  for (ALL_LIST_ELEMENTS_RO (area->circuit_list, cnode, circuit))
     {
-      circuit = getdata (cnode);
       if (circuit->state != C_STATE_UP)
 	continue;
       if (!(circuit->circuit_is_type & level))
@@ -752,11 +795,8 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
       if (family == AF_INET)
 	{
 	  prefix.family = AF_INET;
-	  for (ipnode =
-	       (circuit->ip_addrs ? listhead (circuit->ip_addrs) : NULL);
-	       ipnode; nextnode (ipnode))
+          for (ALL_LIST_ELEMENTS_RO (circuit->ip_addrs, ipnode, ipv4))
 	    {
-	      ipv4 = getdata (ipnode);
 	      prefix.u.prefix4 = ipv4->prefix;
 	      prefix.prefixlen = ipv4->prefixlen;
 	      isis_spf_add_local (spftree, VTYPE_IPREACH_INTERNAL, &prefix,
@@ -767,11 +807,8 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
       if (family == AF_INET6)
 	{
 	  prefix.family = AF_INET6;
-	  for (ipnode = (circuit->ipv6_non_link ? listhead
-			 (circuit->ipv6_non_link) : NULL); ipnode;
-	       nextnode (ipnode))
+	  for (ALL_LIST_ELEMENTS_RO (circuit->ipv6_non_link, ipnode, ipv6))
 	    {
-	      ipv6 = getdata (ipnode);
 	      prefix.prefixlen = ipv6->prefixlen;
 	      prefix.u.prefix6 = ipv6->prefix;
 	      isis_spf_add_local (spftree, VTYPE_IP6REACH_INTERNAL,
@@ -790,26 +827,25 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
 	  if (listcount (adj_list) == 0)
 	    {
 	      list_delete (adj_list);
-	      zlog_warn ("ISIS-Spf: no L%d adjacencies on circuit %s",
-			 level, circuit->interface->name);
+	      if (isis->debugs & DEBUG_SPF_EVENTS)
+		zlog_debug ("ISIS-Spf: no L%d adjacencies on circuit %s",
+			    level, circuit->interface->name);
 	      continue;
 	    }
 	  anode = listhead (adj_list);
 	  while (anode)
 	    {
-	      adj = getdata (anode);
+	      adj = listgetdata (anode);
 	      if (!speaks (&adj->nlpids, family))
 		{
-		  nextnode (anode);
+		  anode = listnextnode (anode);
 		  continue;
 		}
 	      switch (adj->sys_type)
 		{
 		case ISIS_SYSTYPE_ES:
 		  isis_spf_add_local (spftree, VTYPE_ES, adj->sysid, adj,
-				      circuit->metrics[level -
-						       1].metric_default,
-				      family);
+				      circuit->te_metric[level - 1], family);
 		  break;
 		case ISIS_SYSTYPE_IS:
 		case ISIS_SYSTYPE_L1_IS:
@@ -817,9 +853,7 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
 		  vertex =
 		    isis_spf_add_local (spftree, VTYPE_NONPSEUDO_IS,
 					adj->sysid, adj,
-					circuit->metrics[level -
-							 1].metric_default,
-					family);
+					circuit->te_metric[level - 1], family);
 		  memcpy (lsp_id, adj->sysid, ISIS_SYS_ID_LEN);
 		  LSP_PSEUDO_ID (lsp_id) = 0;
 		  LSP_FRAGMENT (lsp_id) = 0;
@@ -834,7 +868,7 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
 		default:
 		  zlog_warn ("isis_spf_preload_tent unknow adj type");
 		}
-	      nextnode (anode);
+	      anode = listnextnode (anode);
 	    }
 	  list_delete (adj_list);
 	  /*
@@ -857,9 +891,8 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
 	    }
 	  else
 	    {
-	      isis_spf_process_pseudo_lsp
-		(spftree, lsp, circuit->metrics[level - 1].metric_default, 0,
-		 family);
+	      isis_spf_process_pseudo_lsp (spftree, lsp,
+				  circuit->te_metric[level - 1], 0, family);
 
 	    }
 	}
@@ -872,17 +905,14 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
 	    {
 	    case ISIS_SYSTYPE_ES:
 	      isis_spf_add_local (spftree, VTYPE_ES, adj->sysid, adj,
-				  circuit->metrics[level - 1].metric_default,
-				  family);
+				  circuit->te_metric[level - 1], family);
 	      break;
 	    case ISIS_SYSTYPE_IS:
 	    case ISIS_SYSTYPE_L1_IS:
 	    case ISIS_SYSTYPE_L2_IS:
 	      if (speaks (&adj->nlpids, family))
 		isis_spf_add_local (spftree, VTYPE_NONPSEUDO_IS, adj->sysid,
-				    adj,
-				    circuit->metrics[level -
-						     1].metric_default,
+				    adj, circuit->te_metric[level - 1],
 				    family);
 	      break;
 	    case ISIS_SYSTYPE_UNKNOWN:
@@ -906,9 +936,9 @@ isis_spf_preload_tent (struct isis_spftree *spftree,
  * The parent(s) for vertex is set when added to TENT list
  * now we just put the child pointer(s) in place
  */
-void
+static void
 add_to_paths (struct isis_spftree *spftree, struct isis_vertex *vertex,
-	      struct isis_area *area)
+	      struct isis_area *area, int level)
 {
 #ifdef EXTREME_DEBUG
   u_char buff[BUFSIZ];
@@ -923,8 +953,8 @@ add_to_paths (struct isis_spftree *spftree, struct isis_vertex *vertex,
   if (vertex->type > VTYPE_ES)
     {
       if (listcount (vertex->Adj_N) > 0)
-	isis_route_create ((struct prefix *) &vertex->N.prefix,
-			   vertex->d_N, vertex->depth, vertex->Adj_N, area);
+	isis_route_create ((struct prefix *) &vertex->N.prefix, vertex->d_N,
+			   vertex->depth, vertex->Adj_N, area, level);
       else if (isis->debugs & DEBUG_SPF_EVENTS)
 	zlog_debug ("ISIS-Spf: no adjacencies do not install route");
     }
@@ -932,7 +962,7 @@ add_to_paths (struct isis_spftree *spftree, struct isis_vertex *vertex,
   return;
 }
 
-void
+static void
 init_spt (struct isis_spftree *spftree)
 {
   spftree->tents->del = spftree->paths->del = (void (*)(void *)) isis_vertex_del;
@@ -943,7 +973,7 @@ init_spt (struct isis_spftree *spftree)
   return;
 }
 
-int
+static int
 isis_run_spf (struct isis_area *area, int level, int family)
 {
   int retval = ISIS_OK;
@@ -952,6 +982,9 @@ isis_run_spf (struct isis_area *area, int level, int family)
   struct isis_spftree *spftree = NULL;
   u_char lsp_id[ISIS_SYS_ID_LEN + 2];
   struct isis_lsp *lsp;
+  struct route_table *table = NULL;
+  struct route_node *rode;
+  struct isis_route_info *rinfo;
 
   if (family == AF_INET)
     spftree = area->spftree[level - 1];
@@ -962,6 +995,23 @@ isis_run_spf (struct isis_area *area, int level, int family)
 
   assert (spftree);
 
+  /* Make all routes in current route table inactive. */
+  if (family == AF_INET)
+    table = area->route_table[level - 1];
+#ifdef HAVE_IPV6
+  else if (family == AF_INET6)
+    table = area->route_table6[level - 1];
+#endif
+
+  for (rode = route_top (table); rode; rode = route_next (rode))
+    {
+      if (rode->info == NULL)
+        continue;
+      rinfo = rode->info;
+
+      UNSET_FLAG (rinfo->flag, ISIS_ROUTE_FLAG_ACTIVE);
+    }
+
   /*
    * C.2.5 Step 0
    */
@@ -977,19 +1027,18 @@ isis_run_spf (struct isis_area *area, int level, int family)
   if (listcount (spftree->tents) == 0)
     {
       zlog_warn ("ISIS-Spf: TENT is empty");
-      spftree->lastrun = time (NULL);
-      return retval;
+      goto out;
     }
 
   while (listcount (spftree->tents) > 0)
     {
       node = listhead (spftree->tents);
-      vertex = getdata (node);
+      vertex = listgetdata (node);
       /* Remove from tent list */
       list_delete_node (spftree->tents, node);
       if (isis_find_vertex (spftree->paths, vertex->N.id, vertex->type))
 	continue;
-      add_to_paths (spftree, vertex, area);
+      add_to_paths (spftree, vertex, area, level);
       if (vertex->type == VTYPE_PSEUDO_IS ||
 	  vertex->type == VTYPE_NONPSEUDO_IS)
 	{
@@ -1018,6 +1067,7 @@ isis_run_spf (struct isis_area *area, int level, int family)
 	}
     }
 
+out:
   thread_add_event (master, isis_route_validate, area, 0);
   spftree->lastrun = time (NULL);
   spftree->pending = 0;
@@ -1139,7 +1189,7 @@ isis_spf_schedule (struct isis_area *area, int level)
 }
 
 #ifdef HAVE_IPV6
-int
+static int
 isis_run_spf6_l1 (struct thread *thread)
 {
   struct isis_area *area;
@@ -1169,7 +1219,7 @@ isis_run_spf6_l1 (struct thread *thread)
   return retval;
 }
 
-int
+static int
 isis_run_spf6_l2 (struct thread *thread)
 {
   struct isis_area *area;
@@ -1188,7 +1238,7 @@ isis_run_spf6_l2 (struct thread *thread)
     }
 
   if (isis->debugs & DEBUG_SPF_EVENTS)
-    zlog_debug ("ISIS-Spf (%s) L2 SPF needed, periodic SPF", area->area_tag);
+    zlog_debug ("ISIS-Spf (%s) L2 SPF needed, periodic SPF.", area->area_tag);
 
   if (area->ipv6_circuits)
     retval = isis_run_spf (area, 2, AF_INET6);
@@ -1253,22 +1303,22 @@ isis_spf_schedule6 (struct isis_area *area, int level)
 }
 #endif
 
-void
+static void
 isis_print_paths (struct vty *vty, struct list *paths)
 {
-  struct listnode *node, *anode;
+  struct listnode *node;
   struct isis_vertex *vertex;
   struct isis_dynhn *dyn, *nh_dyn = NULL;
   struct isis_adjacency *adj;
-#ifdef EXTREME_DEBUG
+#if 0
   u_char buff[255];
-#endif
+#endif /* 0 */
 
   vty_out (vty, "System Id            Metric     Next-Hop"
 	   "             Interface   SNPA%s", VTY_NEWLINE);
-  for (node = listhead (paths); node; nextnode (node))
+
+  for (ALL_LIST_ELEMENTS_RO (paths, node, vertex))
     {
-      vertex = getdata (node);
       if (vertex->type != VTYPE_NONPSEUDO_IS)
 	continue;
       if (memcmp (vertex->N.id, isis->sysid, ISIS_SYS_ID_LEN) == 0)
@@ -1279,23 +1329,22 @@ isis_print_paths (struct vty *vty, struct list *paths)
       else
 	{
 	  dyn = dynhn_find_by_id ((u_char *) vertex->N.id);
-	  anode = listhead (vertex->Adj_N);
-	  adj = getdata (anode);
+	  adj = listgetdata (listhead (vertex->Adj_N));
 	  if (adj)
 	    {
 	      nh_dyn = dynhn_find_by_id (adj->sysid);
 	      vty_out (vty, "%-20s %-10u %-20s %-11s %-5s%s",
 		       (dyn != NULL) ? dyn->name.name :
-		       (u_char *) rawlspid_print ((u_char *) vertex->N.id),
+		       (const u_char *)rawlspid_print ((u_char *) vertex->N.id),
 		       vertex->d_N, (nh_dyn != NULL) ? nh_dyn->name.name :
-		       (u_char *) rawlspid_print (adj->sysid),
+		       (const u_char *)rawlspid_print (adj->sysid),
 		       adj->circuit->interface->name,
 		       snpa_print (adj->snpa), VTY_NEWLINE);
 	    }
 	  else
 	    {
 	      vty_out (vty, "%s              %u %s", dyn ? dyn->name.name :
-		       (u_char *) rawlspid_print (vertex->N.id),
+		       (const u_char *) rawlspid_print (vertex->N.id),
 		       vertex->d_N, VTY_NEWLINE);
 	    }
 	}
@@ -1320,10 +1369,8 @@ DEFUN (show_isis_topology,
   if (!isis->area_list || isis->area_list->count == 0)
     return CMD_SUCCESS;
 
-  for (node = listhead (isis->area_list); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (isis->area_list, node, area))
     {
-      area = getdata (node);
-
       vty_out (vty, "Area %s:%s", area->area_tag ? area->area_tag : "null",
 	       VTY_NEWLINE);
 
@@ -1366,10 +1413,8 @@ DEFUN (show_isis_topology_l1,
   if (!isis->area_list || isis->area_list->count == 0)
     return CMD_SUCCESS;
 
-  for (node = listhead (isis->area_list); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (isis->area_list, node, area))
     {
-      area = getdata (node);
-
       vty_out (vty, "Area %s:%s", area->area_tag ? area->area_tag : "null",
 	       VTY_NEWLINE);
 
@@ -1408,10 +1453,8 @@ DEFUN (show_isis_topology_l2,
   if (!isis->area_list || isis->area_list->count == 0)
     return CMD_SUCCESS;
 
-  for (node = listhead (isis->area_list); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (isis->area_list, node, area))
     {
-      area = getdata (node);
-
       vty_out (vty, "Area %s:%s", area->area_tag ? area->area_tag : "null",
 	       VTY_NEWLINE);
 
