@@ -249,13 +249,6 @@ zprivs_caps_init (struct zebra_privs_t *zprivs)
                        "but no capabilities supplied\n");
     }
 
-  if ( !(zprivs_state.caps = cap_init()) )
-    {
-      fprintf (stderr, "privs_init: failed to cap_init, %s\n", 
-               safe_strerror (errno));
-      exit (1);
-    }
-
   /* we have caps, we have no need to ever change back the original user */
   if (zprivs_state.zuid)
     {
@@ -267,6 +260,13 @@ zprivs_caps_init (struct zebra_privs_t *zprivs)
         }
     }
   
+  if ( !(zprivs_state.caps = cap_init()) )
+    {
+      fprintf (stderr, "privs_init: failed to cap_init, %s\n", 
+               safe_strerror (errno));
+      exit (1);
+    }
+
   if ( cap_clear (zprivs_state.caps) )
     {
       fprintf (stderr, "privs_init: failed to cap_clear, %s\n", 
@@ -348,7 +348,7 @@ static pset_t *
 zcaps2sys (zebra_capabilities_t *zcaps, int num)
 {
   pset_t *syscaps;
-  int i, j = 0, count = 0;
+  int i, j = 0;
   
   if ((syscaps = priv_allocset()) == NULL)
     {
@@ -405,14 +405,14 @@ zprivs_state_caps (void)
   
   if ( (effective = priv_allocset()) == NULL)
     {
-      fprintf (stderr, "%s: failed to get priv_allocset!\n", __func__,
+      fprintf (stderr, "%s: failed to get priv_allocset! %s\n", __func__,
                safe_strerror (errno));
       return ZPRIVS_UNKNOWN;
     }
   
   if (getppriv (PRIV_EFFECTIVE, effective))
     {
-      fprintf (stderr, "%s: failed to get state!%s\n", __func__,
+      fprintf (stderr, "%s: failed to get state! %s\n", __func__,
                safe_strerror (errno));
       result = ZPRIVS_UNKNOWN;
     }
@@ -483,6 +483,19 @@ zprivs_caps_init (struct zebra_privs_t *zprivs)
   /* need either valid or empty sets for both p and i.. */
   assert (zprivs_state.syscaps_i && zprivs_state.syscaps_p);
   
+  /* we have caps, we have no need to ever change back the original user
+   * change real, effective and saved to the specified user.
+   */
+  if (zprivs_state.zuid)
+    {
+      if ( setreuid (zprivs_state.zuid, zprivs_state.zuid) )
+        {
+          fprintf (stderr, "%s: could not setreuid, %s\n", 
+                   __func__, safe_strerror (errno));
+          exit (1);
+        }
+    }
+  
   /* set the permitted set */
   if (setppriv (PRIV_SET, PRIV_PERMITTED, zprivs_state.syscaps_p))
     {
@@ -499,17 +512,6 @@ zprivs_caps_init (struct zebra_privs_t *zprivs)
       exit (1);
     }
 
-  /* we have caps, we have no need to ever change back the original user */
-  if (zprivs_state.zuid)
-    {
-      if ( setreuid (zprivs_state.zuid, zprivs_state.zuid) )
-        {
-          fprintf (stderr, "%s: could not setreuid, %s\n", 
-                   __func__, safe_strerror (errno));
-          exit (1);
-        }
-    }
-  
   /* now clear the effective set and we're ready to go */
   if (setppriv (PRIV_SET, PRIV_EFFECTIVE, empty))
     {
