@@ -46,9 +46,7 @@
 #include "ospfd/ospf_ia.h"
 #include "ospfd/ospf_dump.h"
 
-#define DEBUG
-
-struct ospf_route *
+static struct ospf_route *
 ospf_find_abr_route (struct route_table *rtrs, 
                      struct prefix_ipv4 *abr,
                      struct ospf_area *area)
@@ -70,7 +68,7 @@ ospf_find_abr_route (struct route_table *rtrs,
   return NULL;
 }
 
-void
+static void
 ospf_ia_network_route (struct ospf *ospf, struct route_table *rt,
 		       struct prefix_ipv4 *p, struct ospf_route *new_or,
 		       struct ospf_route *abr_or)
@@ -127,7 +125,7 @@ ospf_ia_network_route (struct ospf *ospf, struct route_table *rt,
     }
 }
 
-void
+static void
 ospf_ia_router_route (struct ospf *ospf, struct route_table *rtrs,
 		      struct prefix_ipv4 *p,
                       struct ospf_route *new_or, struct ospf_route *abr_or)
@@ -196,7 +194,7 @@ ospf_ia_router_route (struct ospf *ospf, struct route_table *rtrs,
 }
 
 
-int
+static int
 process_summary_lsa (struct ospf_area *area, struct route_table *rt,
 		     struct route_table *rtrs, struct ospf_lsa *lsa)
 {
@@ -285,7 +283,7 @@ process_summary_lsa (struct ospf_area *area, struct route_table *rt,
   return 0;
 }
 
-void
+static void
 ospf_examine_summaries (struct ospf_area *area,
 			struct route_table *lsdb_rt,
                         struct route_table *rt,
@@ -305,7 +303,7 @@ ospf_area_is_transit (struct ospf_area *area)
     ospf_full_virtual_nbrs(area); /* Cisco forgets to set the V-bit :( */
 }
 
-void
+static void
 ospf_update_network_route (struct ospf *ospf,
 			   struct route_table *rt, 
                            struct route_table *rtrs,
@@ -434,7 +432,7 @@ ospf_update_network_route (struct ospf *ospf,
     }
 }
 
-void
+static void
 ospf_update_router_route (struct ospf *ospf,
 			  struct route_table *rtrs, 
                           struct summary_lsa *lsa,
@@ -466,13 +464,11 @@ ospf_update_router_route (struct ospf *ospf,
 
   if ((ospf->backbone == NULL) &&
       (ospf->abr_type != OSPF_ABR_SHORTCUT))
-
-     /* no BB area, not Shortcut ABR, exiting */
-     return;
- 
-  or = ospf_find_asbr_route_through_area (rtrs, p, ospf->backbone);
-
-  if (or == NULL)
+     return; /* no BB area, not Shortcut ABR, exiting */
+  
+  /* find the backbone route, if possible */
+  if ((ospf->backbone == NULL)
+      || !(or = ospf_find_asbr_route_through_area (rtrs, p, ospf->backbone)))
     {
       if (ospf->abr_type != OSPF_ABR_SHORTCUT)
 
@@ -533,7 +529,7 @@ ospf_update_router_route (struct ospf *ospf,
     }
 }
 
-int
+static int
 process_transit_summary_lsa (struct ospf_area *area, struct route_table *rt,
 			     struct route_table *rtrs, struct ospf_lsa *lsa)
 {
@@ -591,7 +587,7 @@ process_transit_summary_lsa (struct ospf_area *area, struct route_table *rt,
   return 0;
 }
 
-void
+static void
 ospf_examine_transit_summaries (struct ospf_area *area,
 				struct route_table *lsdb_rt,
                                 struct route_table *rt,
