@@ -212,6 +212,8 @@ ospf_new (void)
 	       "a socket");
       exit(1);
     }
+  new->maxsndbuflen = 0;
+  ospf_adjust_sndbuflen (new, OSPF_SNDBUFLEN_DEFAULT);
   if ((new->ibuf = stream_new(OSPF_MAX_PACKET_SIZE+1)) == NULL)
     {
       zlog_err("ospf_new: fatal error: stream_new(%u) failed allocating ibuf",
@@ -799,7 +801,7 @@ ospf_network_unset (struct ospf *ospf, struct prefix_ipv4 *p,
 int 
 ospf_network_match_iface(struct connected *co, struct prefix *net)
 {
-#define COMPATIBILITY_MODE
+/* #define COMPATIBILITY_MODE */
   /* The old code used to have a special case for PtP interfaces:
 
      if (if_is_pointopoint (co->ifp) && co->destination &&
